/**
 * This file is part of
 * 
 * Repast Simphony MySQL Database Outputter
 *
 * Copyright (C) 2012 Argonne National Laboratory
 * Copyright (C) 2013 Center for Environmental Systems Research, Kassel, Germany
 * 
 * Repast Simphony MySQL Database Outputter is free software; you can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; If not, see <http://www.gnu.org/licenses/>.
 * 
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 13.04.2010
 */
package repast.simphony.data.db.outputter.gui;



import org.pietschy.wizard.PanelWizardStep;

import repast.simphony.data.db.outputter.engine.DbOutputterComponentControllerAction;
import repast.simphony.data.db.outputter.engine.DbOutputterDescriptor;
import repast.simphony.data2.util.DataUtilities;
import repast.simphony.engine.environment.ControllerAction;
import repast.simphony.ui.DescriptorActionUI;
import repast.simphony.ui.plugin.editor.Editor;
import repast.simphony.ui.plugin.editor.OptionsEditorDialog;
import repast.simphony.ui.tree.ScenarioTreeEvent;



/**
 * repast.simphony.data.db
 * 
 * Creates the wizard to edit the descriptor.
 * 
 * @author Sascha Holzhauer
 * @date 13.04.2010
 * 
 */
public class DbOutputterDescriptorActionUI extends DescriptorActionUI<DbOutputterDescriptor> {

	private final DbOutputterComponentControllerAction<DbOutputterDescriptor>	action;

	/**
	 * The dialog's size
	 */
	protected final int														WIDTH_ENLARGE	= 100;

	/**
	 * @param action the {@link ControllerAction} for this DB outputter that is registered.
	 */
	public DbOutputterDescriptorActionUI(DbOutputterComponentControllerAction<DbOutputterDescriptor> action) {
		super(action.getDescriptor());
		this.action = action;
	}

	/**
	 * @see repast.simphony.ui.DescriptorActionUI#getEditor(repast.simphony.ui.tree.ScenarioTreeEvent)
	 */
	@Override
	public Editor getEditor(ScenarioTreeEvent evt) {
		DbOutputterWizard wizard = new DbOutputterWizard(DataUtilities.getDataSetDescriptors(evt.getRegistry(),evt.getContextID()),
				action.getDescriptor());
		OptionsEditorDialog dialog = new OptionsEditorDialog("Db Outputter");
		

		for (PanelWizardStep step : (Iterable<PanelWizardStep>) wizard.getStepsForEditor()) {
			dialog.addContent(step.getName(), null, step, wizard.getWizard());
		}

		return dialog;
	}
}
